import { createSlice, } from '@reduxjs/toolkit';
import { combineEpics } from 'redux-observable';
export const createModule = ({ name, initialState = {}, reducers, selectors, epics, extraReducers, }) => {
    let _initialState = {};
    if (initialState) {
        if (typeof initialState === 'function') {
            _initialState = initialState(_initialState);
        }
        else {
            _initialState = { ..._initialState, ...initialState };
        }
    }
    let _selectors = {};
    if (selectors) {
        if (typeof selectors === 'function') {
            _selectors = selectors(_selectors);
        }
        else {
            _selectors = { ..._selectors, ...selectors };
        }
    }
    let _reducers = {};
    if (reducers) {
        if (typeof reducers === 'function') {
            _reducers = reducers(_reducers);
        }
        else {
            _reducers = { ..._reducers, ...reducers };
        }
    }
    const fixedReducers = Object.keys(_reducers).reduce((_fixedReducers, reducerName) => {
        let ro = _reducers[reducerName];
        if (typeof ro === 'function') {
            ro = {
                reducer: ro,
                prepare: (arg = {}) => {
                    if (Array.isArray(arg)) {
                        return { payload: arg };
                    }
                    else if (typeof arg === 'object') {
                        const { meta, error, ...payload } = arg;
                        return { payload, meta, error };
                    }
                    else {
                        return { payload: arg };
                    }
                },
            };
        }
        return {
            ..._fixedReducers,
            [reducerName]: ro,
        };
    }, {});
    const slice = createSlice({
        name,
        initialState: _initialState,
        reducers: fixedReducers,
        extraReducers,
    });
    let _epics = {};
    if (epics) {
        if (typeof epics === 'function') {
            _epics = epics(_epics);
        }
        else {
            _epics = { ..._epics, ...epics };
        }
    }
    const epicList = Object.keys(_epics).map((k) => _epics[k]);
    const epic = combineEpics(...epicList);
    return {
        slice,
        epic,
        name,
        actions: slice.actions,
        reducer: slice.reducer,
        selectors: _selectors,
    };
};
